import 'connect.m.dart';
import 'authentication.m.dart';

import 'package:fis_common/json_convert.dart';

class UploadDeviceDTO extends BaseDTO{
	String? serialNumber;
	String? deviceModel;
	String? deviceType;
	String? deviceSoftwareVersion;
	String? sDKSoftwareVersion;

	UploadDeviceDTO({
		this.serialNumber,
		this.deviceModel,
		this.deviceType,
		this.deviceSoftwareVersion,
		this.sDKSoftwareVersion,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory UploadDeviceDTO.fromJson(Map<String, dynamic> map) {
		return UploadDeviceDTO( 
			serialNumber: map['SerialNumber'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(deviceType != null)
			map['DeviceType'] = deviceType;
		if(deviceSoftwareVersion != null)
			map['DeviceSoftwareVersion'] = deviceSoftwareVersion;
		if(sDKSoftwareVersion != null)
			map['SDKSoftwareVersion'] = sDKSoftwareVersion;
		return map;
	}
}

class CreateDeviceRequest extends UploadDeviceDTO{

	CreateDeviceRequest({
		String? serialNumber,
		String? deviceModel,
		String? deviceType,
		String? deviceSoftwareVersion,
		String? sDKSoftwareVersion,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			serialNumber: serialNumber,
			deviceModel: deviceModel,
			deviceType: deviceType,
			deviceSoftwareVersion: deviceSoftwareVersion,
			sDKSoftwareVersion: sDKSoftwareVersion,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory CreateDeviceRequest.fromJson(Map<String, dynamic> map) {
		return CreateDeviceRequest( 
			serialNumber: map['SerialNumber'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class DeviceExtendInfoDTO extends DeviceInfoDTO{
	String? organizationName;
	String? departmentName;
	bool isOnline;

	DeviceExtendInfoDTO({
		this.organizationName,
		this.departmentName,
		this.isOnline = false,
		String? deviceCode,
		String? serialNumber,
		String? password,
		String? name,
		String? description,
		String? deviceModel,
		String? deviceType,
		String? headPicUrl,
		String? deviceSoftwareVersion,
		String? sDKSoftwareVersion,
		String? organizationCode,
		String? departmentCode,
		String? shortCode,
		bool isAutoShared = false,
		DateTime? lastLoginTime,
		String? systemVersion,
		String? cPUModel,
		String? systemLanguage,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			deviceCode: deviceCode,
			serialNumber: serialNumber,
			password: password,
			name: name,
			description: description,
			deviceModel: deviceModel,
			deviceType: deviceType,
			headPicUrl: headPicUrl,
			deviceSoftwareVersion: deviceSoftwareVersion,
			sDKSoftwareVersion: sDKSoftwareVersion,
			organizationCode: organizationCode,
			departmentCode: departmentCode,
			shortCode: shortCode,
			isAutoShared: isAutoShared,
			lastLoginTime: lastLoginTime,
			systemVersion: systemVersion,
			cPUModel: cPUModel,
			systemLanguage: systemLanguage,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DeviceExtendInfoDTO.fromJson(Map<String, dynamic> map) {
		return DeviceExtendInfoDTO( 
			organizationName: map['OrganizationName'],
			departmentName: map['DepartmentName'],
			isOnline: map['IsOnline'],
			deviceCode: map['DeviceCode'],
			serialNumber: map['SerialNumber'],
			password: map['Password'],
			name: map['Name'],
			description: map['Description'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			headPicUrl: map['HeadPicUrl'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			shortCode: map['ShortCode'],
			isAutoShared: map['IsAutoShared'],
			lastLoginTime: map['LastLoginTime'] != null ? DateTime.parse(map['LastLoginTime']) : null,
			systemVersion: map['SystemVersion'],
			cPUModel: map['CPUModel'],
			systemLanguage: map['SystemLanguage'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		if(departmentName != null)
			map['DepartmentName'] = departmentName;
		map['IsOnline'] = isOnline;
		return map;
	}
}

class GetDeviceRequest extends TokenRequest{
	String? deviceCode;

	GetDeviceRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetDeviceRequest.fromJson(Map<String, dynamic> map) {
		return GetDeviceRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class PageCollection<T> {
	int currentPage;
	int pageSize;
	int dataCount;
	List<T>? pageData;

	PageCollection({
		this.currentPage = 0,
		this.pageSize = 0,
		this.dataCount = 0,
		this.pageData,
	});

	factory PageCollection.fromJson(Map<String, dynamic> map) {
		List<T> pageDataList = [];
		if (map['PageData'] != null) {
			pageDataList.addAll(
					(map['PageData'] as List).map((e) => FJsonConvert.fromJson<T>(e)!));
		}
		return PageCollection( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			dataCount: map['DataCount'],
			pageData: pageDataList,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		map['DataCount'] = dataCount;
		if(pageData != null)
			map['PageData'] = pageData;
		return map;
	}
}

class PageFilterRequest extends BaseRequest{
	int currentPage;
	int pageSize;
	Map<String,String>? filter;
	bool isFuzzy;

	PageFilterRequest({
		this.currentPage = 0,
		this.pageSize = 0,
		this.filter,
		this.isFuzzy = false,
	}) : super(
		);

	factory PageFilterRequest.fromJson(Map<String, dynamic> map) {
		return PageFilterRequest( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'] != null ? map['Filter'].cast<String,String>() : null,
			isFuzzy: map['IsFuzzy'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		if(filter != null)
			map['Filter'] = filter;
		map['IsFuzzy'] = isFuzzy;
		return map;
	}
}

class BindDeviceRequest extends TokenRequest{
	String? serialNumber;
	String? name;
	String? description;
	String? headPicUrl;
	String? organizationCode;
	String? departmentCode;
	String? shortCode;
	bool isAutoShared;

	BindDeviceRequest({
		this.serialNumber,
		this.name,
		this.description,
		this.headPicUrl,
		this.organizationCode,
		this.departmentCode,
		this.shortCode,
		this.isAutoShared = false,
		String? token,
	}) : super(
			token: token,
		);

	factory BindDeviceRequest.fromJson(Map<String, dynamic> map) {
		return BindDeviceRequest( 
			serialNumber: map['SerialNumber'],
			name: map['Name'],
			description: map['Description'],
			headPicUrl: map['HeadPicUrl'],
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			shortCode: map['ShortCode'],
			isAutoShared: map['IsAutoShared'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		if(headPicUrl != null)
			map['HeadPicUrl'] = headPicUrl;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		map['IsAutoShared'] = isAutoShared;
		return map;
	}
}

class AlterDeviceRequest extends TokenRequest{
	String? deviceCode;
	String? serialNumber;
	String? name;
	String? description;
	String? headPicUrl;
	String? organizationCode;
	String? departmentCode;
	bool isAutoShared;

	AlterDeviceRequest({
		this.deviceCode,
		this.serialNumber,
		this.name,
		this.description,
		this.headPicUrl,
		this.organizationCode,
		this.departmentCode,
		this.isAutoShared = false,
		String? token,
	}) : super(
			token: token,
		);

	factory AlterDeviceRequest.fromJson(Map<String, dynamic> map) {
		return AlterDeviceRequest( 
			deviceCode: map['DeviceCode'],
			serialNumber: map['SerialNumber'],
			name: map['Name'],
			description: map['Description'],
			headPicUrl: map['HeadPicUrl'],
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			isAutoShared: map['IsAutoShared'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		if(headPicUrl != null)
			map['HeadPicUrl'] = headPicUrl;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		map['IsAutoShared'] = isAutoShared;
		return map;
	}
}

enum DictionaryTypeEnum {
	DeviceModel,
	DeviceType,
}

class CreateDictionaryItemRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;
	String? dictionaryItemValue;
	String? parentCode;

	CreateDictionaryItemRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.dictionaryItemValue,
		this.parentCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateDictionaryItemRequest.fromJson(Map<String, dynamic> map) {
		return CreateDictionaryItemRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			dictionaryItemValue: map['DictionaryItemValue'],
			parentCode: map['ParentCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		if(dictionaryItemValue != null)
			map['DictionaryItemValue'] = dictionaryItemValue;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		return map;
	}
}

class DictionaryDTO extends BaseDTO{
	String? dictionaryCode;
	DictionaryTypeEnum dictionaryType;
	String? value;
	String? parentCode;

	DictionaryDTO({
		this.dictionaryCode,
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.value,
		this.parentCode,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DictionaryDTO.fromJson(Map<String, dynamic> map) {
		return DictionaryDTO( 
			dictionaryCode: map['DictionaryCode'],
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			value: map['Value'],
			parentCode: map['ParentCode'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(dictionaryCode != null)
			map['DictionaryCode'] = dictionaryCode;
		map['DictionaryType'] = dictionaryType.index;
		if(value != null)
			map['Value'] = value;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		return map;
	}
}

class FindDeviceModelItemsRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;
	String? deviceTypeCode;

	FindDeviceModelItemsRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.deviceTypeCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDeviceModelItemsRequest.fromJson(Map<String, dynamic> map) {
		return FindDeviceModelItemsRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			deviceTypeCode: map['DeviceTypeCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		if(deviceTypeCode != null)
			map['DeviceTypeCode'] = deviceTypeCode;
		return map;
	}
}

class FindDeviceTypeItemsRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;

	FindDeviceTypeItemsRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDeviceTypeItemsRequest.fromJson(Map<String, dynamic> map) {
		return FindDeviceTypeItemsRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		return map;
	}
}

class CreateShareDeviceToUserRequest extends TokenRequest{
	List<String>? userCodes;
	String? deviceCode;

	CreateShareDeviceToUserRequest({
		this.userCodes,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateShareDeviceToUserRequest.fromJson(Map<String, dynamic> map) {
		return CreateShareDeviceToUserRequest( 
			userCodes: map['UserCodes'] != null ? map['UserCodes'].cast<String>().toList() : null,
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCodes != null)
			map['UserCodes'] = userCodes;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class DeleteShareDeviceToUserRequest extends TokenRequest{
	List<String>? userCodes;
	String? deviceCode;

	DeleteShareDeviceToUserRequest({
		this.userCodes,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteShareDeviceToUserRequest.fromJson(Map<String, dynamic> map) {
		return DeleteShareDeviceToUserRequest( 
			userCodes: map['UserCodes'] != null ? map['UserCodes'].cast<String>().toList() : null,
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCodes != null)
			map['UserCodes'] = userCodes;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class RemoveDeviceRelevancyRequest extends TokenRequest{
	String? organizationCode;
	String? departmentCode;
	String? deviceCode;

	RemoveDeviceRelevancyRequest({
		this.organizationCode,
		this.departmentCode,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoveDeviceRelevancyRequest.fromJson(Map<String, dynamic> map) {
		return RemoveDeviceRelevancyRequest( 
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class FindDevicesByOrganizationCodeRequest extends TokenRequest{
	String? organizationCode;

	FindDevicesByOrganizationCodeRequest({
		this.organizationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDevicesByOrganizationCodeRequest.fromJson(Map<String, dynamic> map) {
		return FindDevicesByOrganizationCodeRequest( 
			organizationCode: map['OrganizationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		return map;
	}
}

class SelectItemDTO {
	String? key;
	String? value;

	SelectItemDTO({
		this.key,
		this.value,
	});

	factory SelectItemDTO.fromJson(Map<String, dynamic> map) {
		return SelectItemDTO( 
			key: map['Key'],
			value: map['Value'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(key != null)
			map['Key'] = key;
		if(value != null)
			map['Value'] = value;
		return map;
	}
}


