import 'connect.m.dart';
import 'authentication.m.dart';

class BaseRoleDTO extends BaseDTO{
	String? roleCode;
	String? roleName;
	String? description;

	BaseRoleDTO({
		this.roleCode,
		this.roleName,
		this.description,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory BaseRoleDTO.fromJson(Map<String, dynamic> map) {
		return BaseRoleDTO( 
			roleCode: map['RoleCode'],
			roleName: map['RoleName'],
			description: map['Description'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(roleCode != null)
			map['RoleCode'] = roleCode;
		if(roleName != null)
			map['RoleName'] = roleName;
		if(description != null)
			map['Description'] = description;
		return map;
	}
}

enum RoleShowTypeEnum {
	NotShow,
	ISShow,
}

enum RoleQualificationEnum {
	NoNeed,
	ID,
	DocLicense,
	Both,
}

class RoleDTO extends BaseRoleDTO{
	RoleShowTypeEnum roleShowType;
	String? iConUrl;
	String? colorStart;
	String? colorEnd;
	RoleQualificationEnum roleQualification;
	String? userGroupCode;

	RoleDTO({
		this.roleShowType = RoleShowTypeEnum.NotShow,
		this.iConUrl,
		this.colorStart,
		this.colorEnd,
		this.roleQualification = RoleQualificationEnum.NoNeed,
		this.userGroupCode,
		String? roleCode,
		String? roleName,
		String? description,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			roleCode: roleCode,
			roleName: roleName,
			description: description,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory RoleDTO.fromJson(Map<String, dynamic> map) {
		return RoleDTO( 
			roleShowType: RoleShowTypeEnum.values.firstWhere((e) => e.index == map['RoleShowType']),
			iConUrl: map['IConUrl'],
			colorStart: map['ColorStart'],
			colorEnd: map['ColorEnd'],
			roleQualification: RoleQualificationEnum.values.firstWhere((e) => e.index == map['RoleQualification']),
			userGroupCode: map['UserGroupCode'],
			roleCode: map['RoleCode'],
			roleName: map['RoleName'],
			description: map['Description'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['RoleShowType'] = roleShowType.index;
		if(iConUrl != null)
			map['IConUrl'] = iConUrl;
		if(colorStart != null)
			map['ColorStart'] = colorStart;
		if(colorEnd != null)
			map['ColorEnd'] = colorEnd;
		map['RoleQualification'] = roleQualification.index;
		if(userGroupCode != null)
			map['UserGroupCode'] = userGroupCode;
		return map;
	}
}

class GetRoleByCodeRequest extends TokenRequest{
	String? roleCode;

	GetRoleByCodeRequest({
		this.roleCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetRoleByCodeRequest.fromJson(Map<String, dynamic> map) {
		return GetRoleByCodeRequest( 
			roleCode: map['RoleCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(roleCode != null)
			map['RoleCode'] = roleCode;
		return map;
	}
}

class FindDefaultRolesRequest extends TokenRequest{
	String? organizationCode;

	FindDefaultRolesRequest({
		this.organizationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDefaultRolesRequest.fromJson(Map<String, dynamic> map) {
		return FindDefaultRolesRequest( 
			organizationCode: map['OrganizationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		return map;
	}
}

class FindAuthenticationRolesRequest extends TokenRequest{

	FindAuthenticationRolesRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory FindAuthenticationRolesRequest.fromJson(Map<String, dynamic> map) {
		return FindAuthenticationRolesRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}


